#!/usr/bin/env python
#
# This file is part of vospace-transfer-service
# Copyright (C) 2021 Istituto Nazionale di Astrofisica
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys
import time

from data_rpc_server import DataRPCServer
from group_rw_rpc_server import GroupRwRPCServer
from import_rpc_server import ImportRPCServer
from job_rpc_server import JobRPCServer
from storage_rpc_server import StorageRPCServer
from user_rpc_server import UserRPCServer


class CliHandler(object):

    def __init__(self, host, port, db):
        self.host = host
        self.port = port
        self.db = db
        self.rpcServerList = []

    def addRPCServer(self, srvType, rpcQueue):
        if srvType == "data":
            self.rpcServerList.append(DataRPCServer(self.host, self.port, self.db, rpcQueue))
        elif srvType == "group_rw":
            self.rpcServerList.append(GroupRwRPCServer(self.host, self.port, self.db, rpcQueue))
        elif srvType == "user":
            self.rpcServerList.append(UserRPCServer(self.host, self.port, self.db, rpcQueue))
        elif srvType == "import":
            self.rpcServerList.append(ImportRPCServer(self.host, self.port, self.db, rpcQueue))
        elif srvType == "storage":
            self.rpcServerList.append(StorageRPCServer(self.host, self.port, self.db, rpcQueue))
        elif srvType == "job":
            self.rpcServerList.append(JobRPCServer(self.host, self.port, self.db, rpcQueue))
        else:
            sys.exit(f"FATAL: unknown server type {srvType}.")

    def start(self):
        for srv in self.rpcServerList:
            srv.start()
        running = False
        while not running:            
            time.sleep(1)
            running = True
            for srv in self.rpcServerList:
                running &= srv.is_alive()
